@@ -1,4 +1,4 @@
-#!/usr/pkg/bin/python3.10
+#!/usr/pkg/bin/python3.11
 # coding=utf-8
 #
 # Copyright (C) 2005,2007,2008 Aaron Spike, aaron@ekips.org
@@ -90,7 +90,11 @@ class DxfOutlines(inkex.OutputExtension):
         pars.add_argument("--tab")
         pars.add_argument("-R", "--ROBO", type=inkex.Boolean, default=False)
         pars.add_argument("-P", "--POLY", type=inkex.Boolean, default=False)
-        pars.add_argument("--units", default="72./96")  # Points
+        pars.add_argument("-F", "--FLATTENBEZ", type=inkex.Boolean, default=False)
+        pars.add_argument(
+            "--unit_from_document", type=inkex.Boolean, default=True
+        )  # px
+        pars.add_argument("--units", default="px")  # px
         pars.add_argument("--encoding", dest="char_encode", default="latin_1")
         pars.add_argument("--layer_option", default="all")
         pars.add_argument("--layer_name")
@@ -104,9 +108,6 @@ class DxfOutlines(inkex.OutputExtension):
         self.d = [0.0]  # knot vector
         self.poly = [[0.0, 0.0]]  # LWPOLYLINE data
 
-    def save(self, stream):
-        stream.write(b"".join(self.dxf))
-
     def dxf_add(self, str):
         self.dxf.append(str.encode(self.options.char_encode))
 
@@ -276,11 +277,20 @@ class DxfOutlines(inkex.OutputExtension):
 
         # Transforming /after/ superpath is more reliable than before
         # because of some issues with arcs in transformations
-        for sub in node.path.to_superpath().transform(Transform(mat) @ node.transform):
+        path = node.path.to_superpath().transform(Transform(mat) @ node.transform)
+
+        # If Flatten Beziers is enabled, subdivide our beziers and
+        # we'll later just ignore the curve and output flat lines
+        if self.options.FLATTENBEZ:
+            bezier.cspsubdiv(path, 0.1)  # default to most detailed (0.1)
+
+        # Now output the path.
+        for sub in path:
             for i in range(len(sub) - 1):
                 s = sub[i]
                 e = sub[i + 1]
-                if s[1] == s[2] and e[0] == e[1]:
+                # If flattening beziers, ignore curves and output flat lines
+                if (s[1] == s[2] and e[0] == e[1]) or self.options.FLATTENBEZ:
                     if self.options.POLY:
                         self.LWPOLY_line([s[1], e[1]])
                     else:
@@ -344,7 +354,6 @@ class DxfOutlines(inkex.OutputExtension):
             try:
                 if isinstance(node, Group):
                     self.process_group(node)
-                    inkex.errormsg(node.get_id())
                 elif isinstance(node, Use):
                     self.process_clone(node)
                 else:
@@ -358,7 +367,7 @@ class DxfOutlines(inkex.OutputExtension):
         if trans:
             self.groupmat.pop()
 
-    def effect(self):
+    def save(self, stream):
         # Warn user if name match field is empty
         if (
             self.options.layer_option
@@ -372,6 +381,9 @@ class DxfOutlines(inkex.OutputExtension):
                 )
             )
 
+        if len(self.svg.xpath("//svg:use|//svg:flowRoot|//svg:text")) > 0:
+            self.preprocess(["flowRoot", "text"])
+
         # Split user layer data into a list: "layerA,layerb,LAYERC" becomes ["layera", "layerb", "layerc"]
         if self.options.layer_name:
             self.options.layer_name = self.options.layer_name.lower().split(",")
@@ -380,8 +392,15 @@ class DxfOutlines(inkex.OutputExtension):
         #              NURB Curves: A Guide for the Uninitiated By Philip J. Schneider
         #              The NURBS Book By Les Piegl and Wayne Tiller (Springer, 1995)
         # self.dxf_add("999\nDXF created by Inkscape\n")  # Some programs do not take comments in DXF files (KLayout 0.21.12 for example)
+        if self.options.unit_from_document:
+            unit = self.svg.document_unit
+        else:
+            unit = self.options.units
         with open(self.get_resource("dxf14_header.txt"), "r") as fhl:
-            self.dxf_add(fhl.read())
+            header = fhl.read()
+            unit_map = {"px": 0, "in": 1, "ft": 2, "mm": 4, "cm": 5, "m": 6}
+            header = header.replace("<unit specifier>", str(unit_map.get(unit, 0)))
+            self.dxf_add(header)
         for node in self.svg.xpath("//svg:g"):
             if isinstance(node, Layer):
                 layer = node.label
@@ -407,21 +426,12 @@ class DxfOutlines(inkex.OutputExtension):
         with open(self.get_resource("dxf14_style.txt"), "r") as fhl:
             self.dxf_add(fhl.read())
 
-        scale = eval(self.options.units)
-        if not scale:
-            scale = 25.4 / 96  # if no scale is specified, assume inch as baseunit
-        scale /= self.svg.unittouu("1px")
-        h = self.svg.viewbox_height
-        doc = self.document.getroot()
-        # process viewBox height attribute to correct page scaling
-        viewBox = doc.get("viewBox")
-        if viewBox:
-            viewBox2 = viewBox.split(",")
-            if len(viewBox2) < 4:
-                viewBox2 = viewBox.split(" ")
-            scale *= h / self.svg.unittouu(self.svg.add_unit(viewBox2[3]))
-        self.groupmat = [[[scale, 0.0, 0.0], [0.0, -scale, h * scale]]]
-        self.process_group(doc)
+        # Set toplevel transform
+        scale = self.svg.inkscape_scale
+        self.groupmat = [
+            [[scale, 0.0, 0.0], [0.0, -scale, self.svg.viewbox_height * scale]]
+        ]
+        self.process_group(self.svg)
         if self.options.ROBO:
             self.ROBO_output()
         if self.options.POLY:
@@ -438,6 +448,8 @@ class DxfOutlines(inkex.OutputExtension):
                 if layer not in self.layernames:
                     inkex.errormsg(_("Warning: Layer '{}' not found!").format(layer))
 
+        stream.write(b"".join(self.dxf))
+
 
 if __name__ == "__main__":
     DxfOutlines().run()
