@@ -1,4 +1,4 @@
-#!/usr/pkg/bin/python3.10
+#!/usr/pkg/bin/python3.11
 # coding=utf-8
 #
 # Copyright (C) 2011 Felipe Correa da Silva Sanches <juca@members.fsf.org>
@@ -29,7 +29,7 @@ class LayersToSvgFont(inkex.EffectExtension):
     def guideline_value(self, label, index):
         for guide in self.svg.namedview.get_guides():
             if guide.label == label:
-                return guide.point[index]
+                return guide.raw_position[index]
         return 0
 
     def flip_cordinate_system(self, path, emsize, baseline):
@@ -65,9 +65,12 @@ class LayersToSvgFont(inkex.EffectExtension):
             label = group.label
             if "GlyphLayer-" in label:
                 unicode_char = label.split("GlyphLayer-")[1]
-                glyph = font.get_or_create(
-                    "svg:glyph[@unicode='{}']".format(unicode_char), Glyph
-                )
+                if unicode_char == "'":
+                    glyph = font.get_or_create("""svg:glyph[@unicode="'"]""", Glyph)
+                else:
+                    glyph = font.get_or_create(
+                        "svg:glyph[@unicode='{}']".format(unicode_char), Glyph
+                    )
                 glyph.set("unicode", unicode_char)
 
                 ############################
